/**
 * Data_info.h
 *
 * Version :   0.2
 * Author  :   Junming Yin
 * 
 * (c) UC Berkeley, 2010
 * 
 */


#ifndef GUARD_Data_info
#define GUARD_Data_info

#include <gsl/gsl_rng.h>          		// for gsl global generator
#include <vector> 
#include <string> 

#define MAX_LENGTH 1024

enum {ESTIMATION=0, LIKELIHOOD=1};

typedef std::vector< std::vector<int> >  intMatrix;

typedef struct locus {
  int index;             			// the index of locus
  double dist;           			// the relative distance of locus (to the left)
} LOCUS;


class Data_info {
 public:
  int N;                         	        // number of haplotypes  
  int L;                            	        // number of loci  
  long int divisor;                 	        // divisor
  int nOrder;                       	        // number of orderings
  int nRep;                         	        // number of repetitions
  double theta;                     	        // mutation rate, either from user input, or computed as Watterson's estimator               

  std::vector<LOCUS> loci;          	        // vector of loci information
  intMatrix haplotypes;             	        // haplotype matrix, size of N*L
  std::vector<intMatrix> orderings; 	        // vector of orderings (length nRep), each of which is size of nOrder*N;	

  // constructors
  Data_info();				        // construct an empty object
  Data_info(char*, char*, int, int, double);    // construct by reading from a file 	
	
  // member functions, read intMatrix from ifstream
  void readMatrix(char*, std::ifstream&, intMatrix&, int&, int&);         	

};


/****************************
     gsl global generator
****************************/

extern const gsl_rng_type* T;
extern gsl_rng* rg;                     

/****************************
     command line options
****************************/
extern std::string command;
extern char *in_file;
extern char *out_file;

extern bool v_flag;
extern unsigned long seed;

extern char *order_file;
extern int n_order;
extern int n_rep;

extern bool mle_flag;
extern bool rho_flag;
extern bool gamma_flag;
extern bool tl_flag;

extern double initial_theta;
extern double initial_rho;
extern double initial_gamma;
extern double initial_tl;

/****************************
    accessory functions 
 ****************************/

int parseCommandline(int argc, char** argv, int flag);
void printHelp(int flag);

#endif
