/**
 * overpaintLike.cc
 *
 * Version :   0.2
 * Author  :   Junming Yin
 * 
 * (c) UC Berkeley, 2010
 * 
 */


#include <iostream>		
#include <fstream>					
#include "Data_info.h"
#include "PAC_GeneCo.h"

using namespace std;


int main(int argc, char** argv)
{

  if (parseCommandline(argc, argv, LIKELIHOOD) == 1)
    exit(1);

  // overpaintLike does not allow -mle option, except -mle 0 0 0
  if  (mle_flag == 1)  {
    cout << "overpaintLike (Error): option -mle is not allowed. Please use overpaintEst instead." << endl;
    exit(1);
  }

  // global random number generator via gsl
  gsl_rng_env_setup();
  T = gsl_rng_default;
  rg = gsl_rng_alloc(T);
  gsl_rng_set(rg, seed);
  
  // create a Data_info object
  Data_info data(in_file, order_file, n_order, n_rep, initial_theta);
  initial_theta = data.theta;
  n_rep = data.nRep;
  n_order = data.nOrder;

  // output the orderings into infile_orderxxx.txt
  if(order_file == '\0') {
    char file[MAX_LENGTH];
    for(int i = 0; i < n_rep; ++i)  {
      sprintf(file, "%s_order%d.txt", in_file, i + 1);
      ofstream order_s(file);
      for(int j = 0; j < n_order; ++j) {
        for(int k = 0; k < data.N; ++k) 
          order_s << data.orderings[i][j][k]  << " ";
        order_s << endl;
      }
      order_s.close();
    }	
  }

  // create a PAC_GeneCo object, PAC_GeneCo prog(data);	
  PAC_GeneCo *prog = new PAC_GeneCo(data);   

  string output = command; 
  output += string(command.size(), '-');
  output += string("\n"); 
  char line[MAX_LENGTH];


  // compute the loglikelihoods
  vector<double> logLiks;
  logLiks.clear();


  // check validity of initial values of rho, gamma and tl
  if( initial_gamma == 0.0 ) {  
    if( initial_rho == 0.0) {   // invalid
      cout << "overpaintLike (Error): at least one of values for rho and gamma have to be non-zero!" << endl;
      exit(1);
    }
    else {                      // initial_rho != 0, PAC with crossovers ONLY
      if ( initial_tl != 0) {
        cout << "overpaintLike (Warning): track length -tl is ignored" << endl;
        initial_tl = 0;
      }

      sprintf(line, "Computing the log-likelihood at (%.5f, 0.0, 0.0) (averaged over %d orders)\n\n", initial_rho, n_order);
      output += string(line);
      output += string("repetition \t\t log-likelihood \n");
      cout << output; 

      prog->setMode(0);
      for(int r = 0; r < n_rep; r++) {
        prog->setCurrentRep(r);
        logLiks.push_back(prog->logLike0(initial_rho));
        // output the result to the terminal
        printf("%6d \t\t\t %12.5f\n", r + 1, logLiks[r]);
        sprintf(line, "%6d \t\t\t %12.5f\n", r + 1, logLiks[r]);
        output += string(line);
      } // end r
      cout << endl;
      output += string("\n");
    }
  }  // end if
  else {    	                // initial_gamma != 0
    if (initial_tl == 0.0) {    // invalid
      cout << "overpaintLike: value of track length -tl has to be non-zero!" << endl;
      exit(1);
    }
    else {                      // PAC with gene-conversions

      sprintf(line, "Computing the log-likelihood at (%.5f, %.5f, %.5f) (averaged over %d orders)\n\n", initial_rho, initial_gamma, initial_tl, n_order);
      output += string(line);
      output += string("repetition \t\t log-likelihood \n");
      cout << output; 

      prog->setMode(1);
      for(int r = 0; r < n_rep; r++) {
        prog->setCurrentRep(r);
        logLiks.push_back(prog->logLike(initial_rho, initial_gamma, 1.0/initial_tl));
        // output the result to the terminal
        printf("%6d \t\t\t %12.5f\n", r + 1, logLiks[r]);
        sprintf(line, "%6d \t\t\t %12.5f\n", r + 1, logLiks[r]);
        output += string(line);
      } // end r
      cout << endl;
      output += string("\n");
    } // end else
  } // end else

  delete prog;

  // output the result into out_file
  if(out_file != '\0') {
    ofstream out_s(out_file);
    out_s << output;
    out_s.close();
  }


  return 0;
} // end main

